// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package datastore

import (
	"context"
	"time"

	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/core"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/log"
	"github.com/redis/go-redis/v9"
)

type RedisCache struct {
	redis *redis.Client
}

func NewCache(opt interface{}) core.Cache {
	redisOpt := opt.(*redis.Options)
	return &RedisCache{redis.NewClient(redisOpt)}
}

func (c *RedisCache) Get(ctx context.Context, key string) (string, error) {
	val, err := c.redis.Get(ctx, key).Result()
	if err == redis.Nil {
		log.DebugLog("redis key not found", "key", key)
		return "", nil
	} else if err != nil {
		log.DebugLog("redis cache get error", "err", err)
		return "", err
	}
	return val, nil
}

func (c *RedisCache) Set(
	ctx context.Context,
	key string,
	value interface{},
	expiration time.Duration,
) error {
	return c.redis.Set(ctx, key, value, expiration).Err()
}

func (c *RedisCache) Close() error {
	return c.redis.Close()
}
