// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package model

import "time"

// {{{ Request Object (from controller to service)

type EventCreateRequest struct {
	Name            string   `json:"name"        binding:"requiered"`
	Description     string   `json:"description"`
	OwnerID         int      `json:"owner_id"    binding:"requiered,number"`
	DefaultCurrency Currency `json:"currency"`
}

// }}}
// {{{ Response View Object (from service to controller)

type EventListResponse struct {
	ID          int
	Name        string
	Description string
	Owner       *UserBaseResponse
	CreatedAt   time.Time
}

type EventInfoResponse struct {
	ID int

	Name        string
	Description string

	TotalAmount Money

	Owner *UserBaseResponse

	CreatedAt time.Time
	UpdatedAt time.Time

	Users []UserBaseResponse
}

// }}}
// {{{ Entity (DB In)

type EventEntity struct {
	ID int

	Name            string
	Description     string
	TotalAmount     int
	DefaultCurrency string
	OwnerID         int

	CreatedAt time.Time
	UpdatedAt time.Time
}

type EventUpdateEntity struct {
	ID          int
	Name        string
	Description string
	CreatedAt   time.Time
	// TODO: maybe I can change owner too
}

// }}}
// {{{ Retrieved (DB out)

type EventRetrieved struct {
	ID int

	Name        string
	Description string

	Users []UserBaseRetrieved

	TotalAmount     Money
	DefaultCurrency Currency
	Owner           *UserBaseRetrieved

	CreatedAt time.Time
	UpdatedAt time.Time
}

type EventListRetrieved struct {
	ID          int
	Name        string
	Description string
	CreatedAt   time.Time
	Owner       *UserBaseRetrieved
}

// }}}
// {{{ DO Domain Object (Contains the domain service)

type Event struct {
	ID int

	Name        string
	Description string

	// lazy get using participation join
	Users []UserDO
	// lazy get
	Expenses []Expense

	TotalAmount     Money
	DefaultCurrency Currency
	Owner           *UserDO

	CreatedAt time.Time
	UpdatedAt time.Time
}

// }}}
