// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package model

import "time"

// {{{ Request  (from controller to service)

type UserCreateRequest struct {
	Email     string `json:"email"      binding:"required,email"`
	FirstName string `json:"first_name" binding:"required"`
	LastName  string `json:"last_name"  binding:"required"`
	Password  string `json:"password"   binding:"required"`
}

type UserExistRequest struct {
	Email    string `json:"email"    binding:"required,email"`
	Password string `json:"password" binding:"required"`
}

// }}}
// {{{ Response View Object (from service to controller)

type UserBaseResponse UserBaseRetrieved

type UserInfoResponse struct {
	// UserBaseResponse
	ID        int    `json:"id"`
	FirstName string `json:"first_name"`
	LastName  string `json:"last_name"`

	Email     string `json:"email"`
	CreatedAt time.Time
	UpdatedAt time.Time
}

// }}}
// {{{ Entity (DB In)

type UserEntity struct {
	ID int

	Email     string
	FirstName string
	LastName  string
	Password  string

	CreatedAt time.Time
	UpdatedAt time.Time
}

// }}}
// {{{ Retrieved (DB out)

type UserBaseRetrieved struct {
	ID        int    `json:"id"`
	FirstName string `json:"first_name"`
	LastName  string `json:"last_name"`
}

// }}}
// {{{ DO Domain Object (Contains the domain service)

// TODO: For now I don't know what to do with this model
type UserDO struct {
	ID int

	Email     string
	FirstName string
	LastName  string
	Password  string

	// Lazy aggregate with the Participation join
	EventIDs []int

	CreatedAt time.Time
	UpdatedAt time.Time
}

// }}}
