// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package registry

import (
	"database/sql"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/controller"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/core"
)

// registry is an implementation of Registry interface.
// It needs a db connection to provide the necessary support.
// It might holds other drivers when the projects grows. For example
// the object needed to connect to Redis or Kafka.
type registry struct {
	db    *sql.DB
	cache core.Cache
}

// Registry returns a new app controller that will be used by main()/run()
// AppController is essentially the struct that holds all the controllers,
// classed by their domains.
type Registry interface {
	NewAppController() controller.AppController
}

// NewRegistry returns a new Registry's implementation.
func NewRegistry(db *sql.DB, cache core.Cache) Registry {
	return &registry{db: db, cache: cache}
}

// NewAppController creates a new AppController with controller struct for
// each domain.
func (r *registry) NewAppController() controller.AppController {
	return controller.AppController{
		User:    r.NewUserController(),
		Admin:   r.NewAdminController(),
		Session: r.NewSessionController(),
	}
}
