// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package repo

import (
	"context"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
)

type EventRepository interface {
	Create(ctx context.Context, evEntity *model.EventEntity, tx any) (*model.EventEntity, error)

	// UpdateEventByID updates the event related information (name, descriptions)
	UpdateEventByID(ctx context.Context, event *model.EventUpdateEntity, tx any) error

	GetByID(ctx context.Context, eventID int, tx any) (*model.EventRetrieved, error)

	// related to events of a user
	ListEventsByUserID(ctx context.Context, userID int, tx any) ([]model.EventListRetrieved, error)

	InsertParticipation(
		ctx context.Context,
		userID, eventID, invitedByUserID int,
		tx any,
	) (*model.ParticipationEntity, error)

	GetParticipation(
		ctx context.Context,
		userID, eventID int,
		tx any,
	) (*model.ParticipationEntity, error)
}

type ExpenseRepository interface {
	DeleteExpense(ctx context.Context, expenseID int, tx any) error
	DeleteTransactionsOfExpense(ctx context.Context, expenseID int, tx any) error
	GetExpenseByID(ctx context.Context, expenseID int, tx any) (*model.ExpenseRetrieved, error)
	InsertExpense(
		ctx context.Context,
		expenseEntity *model.ExpenseEntity,
		tx any,
	) (*model.ExpenseEntity, error)
	ListExpensesByEventID(
		ctx context.Context,
		id int,
		tx any,
	) ([]model.ExpensesListRetrieved, error)
	UpdateExpenseByID(
		ctx context.Context,
		expenseUpdate *model.ExpenseUpdateEntity,
		tx any,
	) (*model.ExpenseEntity, error)
}
