// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package repomock

import (
	"context"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"github.com/pkg/errors"
	"golang.org/x/crypto/bcrypt"
)

var UserTestDummyErr = errors.New("dummy error")

type TestUserRepository struct{}

func (tur *TestUserRepository) Create(
	ctx context.Context,
	u *model.UserEntity,
	tx any,
) (*model.UserEntity, error) {
	user := *u

	user.ID = 123

	if user.Email == "duplicate@error.com" {
		return nil, errors.New("blabla (SQLSTATE 23505)")
	}

	return &user, nil
}

func (tur *TestUserRepository) GetByEmail(
	ctx context.Context,
	email string,
	tx any,
) (*model.UserEntity, error) {
	hashedPwd, _ := bcrypt.GenerateFromPassword([]byte("strongHashed"), 12)
	switch email {
	case "a@b.c":
		return &model.UserEntity{
			ID:       123,
			Email:    "a@b.c",
			Password: string(hashedPwd),
		}, nil
	case "query@error.com":
		return nil, UserTestDummyErr
	case "inexist@error.com":
		return nil, nil
	}

	return nil, UserTestDummyErr
}

func (tur *TestUserRepository) GetByID(
	ctx context.Context,
	id int,
	tx any,
) (*model.UserEntity, error) {
	hashedPwd, _ := bcrypt.GenerateFromPassword([]byte("strongHashed"), 12)
	switch id {
	case 123:
		return &model.UserEntity{
			ID:       123,
			Email:    "a@b.c",
			Password: string(hashedPwd),
		}, nil
	case 456:
		return nil, UserTestDummyErr
	case 789:
		return nil, nil
	}
	return nil, UserTestDummyErr
}
