// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package middleware

import (
	"net/http"
	"testing"

	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/shared"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/test"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
	"github.com/stretchr/testify/assert"
)

func TestRequestID(t *testing.T) {
	r := gin.New()
	r.Use(RequestID())
	var got string
	wanted := "123"

	r.GET("/example", func(c *gin.Context) {
		got = c.GetHeader(shared.XRequestID)
		c.Status(http.StatusOK)
	})

	r.POST("/example", func(c *gin.Context) {
		got = c.GetHeader(shared.XRequestID)
		c.String(http.StatusAccepted, "ok")
	})

	// Test with Request ID
	_ = test.PerformRequest(
		t,
		r,
		"GET",
		"/example?a=100",
		nil,
		test.Header{Key: shared.XRequestID, Value: wanted},
	)
	assert.Equal(t, "123", got)

	res := test.PerformRequest(t, r, "GET", "/example?a=100", nil)
	assert.NotEqual(t, "", got)
	assert.NoError(t, uuid.Validate(got))
	assert.Equal(t, res.Header()[shared.XRequestID][0], got)
}
